@@ -6,16 +6,18 @@ psycopg connection pool base class and functionalities.
 
 from time import monotonic
 from random import random
-from typing import Any, Callable, Dict, Generic, Optional, Tuple
+from typing import Any, Dict, Optional, Tuple, TYPE_CHECKING
 
 from psycopg import errors as e
-from psycopg.abc import ConnectionType
 
 from .errors import PoolClosed
 from ._compat import Counter, Deque
 
+if TYPE_CHECKING:
+    from psycopg._connection_base import BaseConnection
 
-class BasePool(Generic[ConnectionType]):
+
+class BasePool:
     # Used to generate pool names
     _num_pool = 0
 
@@ -36,6 +38,8 @@ class BasePool(Generic[ConnectionType]):
     _CONNECTIONS_ERRORS = "connections_errors"
     _CONNECTIONS_LOST = "connections_lost"
 
+    _pool: Deque["Any"]
+
     def __init__(
         self,
         conninfo: str = "",
@@ -43,14 +47,12 @@ class BasePool(Generic[ConnectionType]):
         kwargs: Optional[Dict[str, Any]],
         min_size: int,
         max_size: Optional[int],
-        open: bool,
         name: Optional[str],
         timeout: float,
         max_waiting: int,
         max_lifetime: float,
         max_idle: float,
         reconnect_timeout: float,
-        reconnect_failed: Optional[Callable[["BasePool[ConnectionType]"], None]],
         num_workers: int,
     ):
         min_size, max_size = self._check_size(min_size, max_size)
@@ -64,8 +66,6 @@ class BasePool(Generic[ConnectionType]):
 
         self.conninfo = conninfo
         self.kwargs: Dict[str, Any] = kwargs or {}
-        self._reconnect_failed: Callable[["BasePool[ConnectionType]"], None]
-        self._reconnect_failed = reconnect_failed or (lambda pool: None)
         self.name = name
         self._min_size = min_size
         self._max_size = max_size
@@ -77,7 +77,7 @@ class BasePool(Generic[ConnectionType]):
         self.num_workers = num_workers
 
         self._nconns = min_size  # currently in the pool, out, being prepared
-        self._pool = Deque[ConnectionType]()
+        self._pool = Deque()
         self._stats = Counter[str]()
 
         # Min number of connections in the pool in a max_idle unit of time.
@@ -95,6 +95,7 @@ class BasePool(Generic[ConnectionType]):
 
         self._opened = False
         self._closed = True
+        self._open_implicit = False
 
     def __repr__(self) -> str:
         return (
@@ -141,7 +142,7 @@ class BasePool(Generic[ConnectionType]):
             else:
                 raise PoolClosed(f"the pool {self.name!r} is not open yet")
 
-    def _check_pool_putconn(self, conn: ConnectionType) -> None:
+    def _check_pool_putconn(self, conn: "BaseConnection[Any]") -> None:
         pool = getattr(conn, "_pool", None)
         if pool is self:
             return
@@ -191,7 +192,7 @@ class BasePool(Generic[ConnectionType]):
         """
         return value * (1.0 + ((max_pc - min_pc) * random()) + min_pc)
 
-    def _set_connection_expiry_date(self, conn: ConnectionType) -> None:
+    def _set_connection_expiry_date(self, conn: "BaseConnection[Any]") -> None:
         """Set an expiry date on a connection.
 
         Add some randomness to avoid mass reconnection.
